function coordinationAnalysis(num_intervals, save_flag)

dname = '.\Data\B_animals_whishaw_task\';
fdir = dir(dname);
flist = {fdir.name};
flist = flist(contains(flist, 'interp'));
ext_flist = flist(contains(flist, 'Extension'));
reach_flist = flist(contains(flist, 'Reach'));

chs = 2 : 11;

iemg_struct = struct();
pk2pk_struct = struct();
pw_struct = struct();

ext_iemg = zeros(num_intervals,length(chs));
reach_iemg = ext_iemg;

minProminence = 0.03;

% Begin loop from 1 to number of time files
for f = 1 : length(ext_flist)
    ext_file = cell2mat(ext_flist(f));
    ext_split = strsplit(ext_file, '_');
    fname{f} = ext_file(1:13);

    reach_file = cell2mat( reach_flist( contains( reach_flist, ...
        cell2mat( ext_split(3) ) ) ) );
    ext_file = [dname ext_file];
    reach_file = [dname reach_file];
    ext_data = csv2struct(ext_file);
    reach_data = csv2struct(reach_file);

% Parse the csv file from the interpolated time intervals for flex/ext section
    fields = fieldnames(ext_data);
    ext_ids = unique(ext_data.id);
    reach_ids = unique(reach_data.id);
    member_mask = ismember(ext_ids, reach_ids);
    ids = ext_ids(member_mask);
    ext_arr = zeros(length(ext_data.id), length(fields));
    for i = 1 : length(fields)
        ext_arr(:,i) = ext_data.(cell2mat(fields(i)));
        reach_arr(:,i) = reach_data.(cell2mat(fields(i))); 
    end
    for i = 1 : length(ids)
        id = ids(i);
        ext_mask = ext_arr(:,1) == id;
        reach_mask = reach_arr(:,1) == id;
        for j = 1 : length(fields)
            ext_id(:,j) = ext_arr(ext_mask,j);
            reach_id(:,j) = reach_arr(reach_mask,j);
        end
% Divide the time intervals into variable sections (i.e. halfs or quarters)
        s_arr = size(ext_id);
        if mod(s_arr(1),num_intervals) ~= 0
            fprintf('invalid interval selection, must be a factor of %d\n',...
                s_arr(1));
            return;
        end
        ext_time = ext_id(:,end);
        reach_time = reach_id(:,end);

        ts_ext = length(ext_time);
        ts_reach = length(reach_time);

        ext_ivals_time = reshape(ext_time, [ts_ext/num_intervals ...
            num_intervals]);
        reach_ivals_time = reshape(reach_time, [ts_reach/num_intervals ...
            num_intervals]);

        ext_intervals = permute(reshape(ext_id', [s_arr(2) s_arr(1)/num_intervals ...
            num_intervals]), [2 1 3]);
        reach_intervals = permute(reshape(reach_id', [s_arr(2) s_arr(1)/num_intervals...
            num_intervals]), [2 1 3]);

% Calculate the IEMG per section, actual time required for each state, flexion
% state and extension state, and also the cocontraction value(?)
        for j = 1 : num_intervals
            ext_ival = squeeze(ext_intervals(:,chs,j));
            reach_ival = squeeze(reach_intervals(:,chs,j));
            ext_iemg(j,:) = calculateIEMG(ext_ival, ext_ivals_time(:,j));
            reach_iemg(j,:) = calculateIEMG(reach_ival, reach_ivals_time(:,j));

            [ext_pk2pk(j,:), ext_npks(j,:)] = calculatePk2Pk(ext_ival, minProminence);
            [reach_pk2pk(j,:), reach_npks(j,:)] = calculatePk2Pk(reach_ival, minProminence);

            ext_pw(j,:) = calculatePulseWidth(ext_ival, minProminence);
            reach_pw(j,:) = calculatePulseWidth(reach_ival, minProminence);
        end
        id_name = sprintf('ID%d', id);
        % iemg_struct.(fname).(id_name).ext = ext_iemg;
        % iemg_struct.(fname).(id_name).reach = reach_iemg;
        ext_itemp = horzcat(ones(num_intervals,1)*id,ext_iemg);
        reach_itemp = horzcat(ones(num_intervals,1)*id,reach_iemg);
        
        ext_ptemp = horzcat(ones(num_intervals,1)*id,ext_pk2pk);
        reach_ptemp = horzcat(ones(num_intervals,1)*id,reach_pk2pk);

        ext_nptemp = horzcat(ones(num_intervals,1)*id,ext_npks);
        reach_nptemp = horzcat(ones(num_intervals,1)*id,reach_npks);

        ext_pwtemp = horzcat(ones(num_intervals,1)*id,ext_pw);
        reach_pwtemp = horzcat(ones(num_intervals,1)*id,reach_pw);

        field = cell2mat(fname(f));
        if ~isfield(iemg_struct,field)
            iemg_struct.(field).ext = ext_itemp;
            iemg_struct.(field).reach = reach_itemp;

            pk2pk_struct.(field).ext = ext_ptemp;
            pk2pk_struct.(field).reach = reach_ptemp;

            npks_struct.(field).ext = ext_nptemp;
            npks_struct.(field).reach = reach_nptemp;

            pw_struct.(field).ext = ext_pwtemp;
            pw_struct.(field).reach = reach_pwtemp;
        else
            ext_hist = iemg_struct.(field).ext; 
            reach_hist = iemg_struct.(field).reach; 
            iemg_struct.(field).ext = vertcat(ext_hist, ext_itemp);
            iemg_struct.(field).reach = vertcat(reach_hist, reach_itemp);

            ext_hist = pk2pk_struct.(field).ext; 
            reach_hist = pk2pk_struct.(field).reach; 
            pk2pk_struct.(field).ext = vertcat(ext_hist, ext_ptemp);
            pk2pk_struct.(field).reach = vertcat(reach_hist, reach_ptemp);

            ext_hist = npks_struct.(field).ext; 
            reach_hist = npks_struct.(field).reach; 
            npks_struct.(field).ext = vertcat(ext_hist, ext_nptemp);
            npks_struct.(field).reach = vertcat(reach_hist, reach_nptemp);

            ext_hist = pw_struct.(field).ext; 
            reach_hist = pw_struct.(field).reach; 
            pw_struct.(field).ext = vertcat(ext_hist, ext_pwtemp);
            pw_struct.(field).reach = vertcat(reach_hist, reach_pwtemp);
        end
    end

% Save the information into a struct for flexion and extension respectively

% End loop
end

% Plot the iEMG confidence interval
field = cell2mat(fname(1));
ext1_ = iemg_struct.(field).ext;
ext1_mu = getNthRowMean(ext1_, num_intervals);
ext1_sd = getNthRowSD(ext1_, num_intervals);
reach1_ = iemg_struct.(field).reach;
reach1_mu = getNthRowMean(reach1_, num_intervals);
reach1_sd = getNthRowSD(reach1_, num_intervals);

field = cell2mat(fname(2));
ext2_ = iemg_struct.(field).ext;
ext2_mu = getNthRowMean(ext2_, num_intervals);
reach2_ = iemg_struct.(field).reach;
reach2_mu = getNthRowMean(reach2_, num_intervals);
reach2_sd = getNthRowSD(reach2_, num_intervals);

durations = csv2struct([dname 'durations.csv']);
f = 1;
figure(f);
plot(durations.ext_time(1:6), durations.reach_time(1:6),'b*', ...
    durations.ext_time(7:end), durations.reach_time(7:end),'r*');
xlabel('extension time (s)');
ylabel('reach time (s)');
legend(fname);

ext = pw_struct.(fname{1}).ext;
reach = pw_struct.(fname{1}).reach;

ext_mu = mean(ext(:,2:end),1);
reach_mu = mean(reach(:,2:end),1);
ext_sd = std(ext(:,2:end),1);
reach_sd = std(reach(:,2:end),1);

ext = npks_struct.(fname{2}).ext;
reach = npks_struct.(fname{2}).reach;

ext_mu(2,:) = mean(ext(:,2:end),1);
reach_mu(2,:) = mean(reach(:,2:end),1);
ext_sd(2,:) = std(ext(:,2:end),1);
reach_sd(2,:) = std(reach(:,2:end),1);

for s = 1 : 10
    figure(f + 1);
    subplot(5,2,s);
    errorbar(ext_mu(:,s),ext_sd(:,s),'*')
    title(['Ch: ' num2str(s)]);
    if s == 1
        title({'extension', ['Ch' num2str(s)]} );
        ylabel('PW');
    end
    set(gca,'XLim', [0 3], 'xtick', [1 2], 'xticklabel', {'B07 BR', 'B16 ND'})

    figure(f + 2);
    subplot(5,2,s);
    errorbar(reach_mu(:,s),reach_sd(:,s),'*')
    title(['Ch: ' num2str(s)]);
    if s == 1
        ylabel('PW');
        title({'reach', ['Ch' num2str(s)]} );
    end
    set(gca,'XLim', [0 3], 'xtick', [1 2], 'xticklabel', {'B07 BR', 'B16 ND'})
end
fig = figure(2);
fig.Position = [0 0 2480 3508];
if save_flag
    saveFig2PDF('.\Figures\','_','CI_pw_ext',2)
end

fig = figure(3);
fig.Position = [0 0 2480 3508];
if save_flag
    saveFig2PDF('.\Figures\','_','CI_pw_rea',3)
end

f = 4;
figure(f);
ch = 3;
br1_ext = iemg_struct.WT_W06_B07_BR.ext(:,ch+1);
br2_ext = iemg_struct.WT_W06_B16_ND.ext(:,ch+1);
br1_rea = iemg_struct.WT_W06_B07_BR.reach(:,ch+1);
br2_rea = iemg_struct.WT_W06_B16_ND.reach(:,ch+1);
hold off;
for a = 1 : num_intervals : max([length(br1_ext) length(br2_ext)])
    subplot(211);
    plot(br1_ext(a:a+num_intervals-1),'b');
    hold on;
    if a < length(br2_ext)
        plot(br2_ext(a:a+num_intervals-1),'r');
    end
    subplot(212);
    plot(br1_rea(a:a+num_intervals-1),'b');
    hold on;
    if a < length(br2_rea)
        plot(br2_rea(a:a+num_intervals-1),'r');
    end
end
subplot(211); title('iemg extension - ch3');
subplot(212); title('iemg reach - ch3');
fig = figure(f);
fig.Position = [0 0 2480 3508];
if save_flag
    saveFig2PDF('.\Figures\','_','iemg_ch3',f)
end

f = f + 1;
figure(f);
ch = 3;
br1_ext = pk2pk_struct.WT_W06_B07_BR.ext(:,ch+1);
br2_ext = pk2pk_struct.WT_W06_B16_ND.ext(:,ch+1);
br1_rea = pk2pk_struct.WT_W06_B07_BR.reach(:,ch+1);
br2_rea = pk2pk_struct.WT_W06_B16_ND.reach(:,ch+1);
hold off;
for a = 1 : num_intervals : max([length(br1_ext) length(br2_ext)])
    subplot(211);
    plot(br1_ext(a:a+num_intervals-1),'b');
    hold on;
    if a < length(br2_ext)
        plot(br2_ext(a:a+num_intervals-1),'r');
    end
    subplot(212);
    plot(br1_rea(a:a+num_intervals-1),'b');
    hold on;
    if a < length(br2_rea)
        plot(br2_rea(a:a+num_intervals-1),'r');
    end
end
subplot(211); title('pk2pk extension - ch3');
subplot(212); title('pk2pk reach - ch3');
legend({'B07 BR','B16 ND'});
fig = figure(f);
fig.Position = [0 0 2480 3508];
if save_flag
    saveFig2PDF('.\Figures\','_','pk2pk_ch3',f)
end

f = f + 1;
figure(f);
br1_ext = npks_struct.WT_W06_B07_BR.ext(:,ch+1);
br2_ext = npks_struct.WT_W06_B16_ND.ext(:,ch+1);
br1_rea = npks_struct.WT_W06_B07_BR.reach(:,ch+1);
br2_rea = npks_struct.WT_W06_B16_ND.reach(:,ch+1);
hold off;
for a = 1 : num_intervals : max([length(br1_ext) length(br2_ext)])
    subplot(211);
    plot(br1_ext(a:a+num_intervals-1),'b');
    hold on;
    if a < length(br2_ext)
        plot(br2_ext(a:a+num_intervals-1),'r');
    end
    subplot(212);
    plot(br1_rea(a:a+num_intervals-1),'b');
    hold on;
    if a < length(br2_rea)
        plot(br2_rea(a:a+num_intervals-1),'r');
    end
end
subplot(211); title('npks extension - ch3');
subplot(212); title('npks reach - ch3');
legend({'B07 BR','B16 ND'});
fig = figure(f);
fig.Position = [0 0 2480 3508];
if save_flag
    saveFig2PDF('.\Figures\','_','npks_ch3',f)
end

f = f + 1;
fig = figure(f);
br1_ext = pw_struct.WT_W06_B07_BR.ext(:,ch+1);
br2_ext = pw_struct.WT_W06_B16_ND.ext(:,ch+1);
br1_rea = pw_struct.WT_W06_B07_BR.reach(:,ch+1);
br2_rea = pw_struct.WT_W06_B16_ND.reach(:,ch+1);
hold off;
for a = 1 : num_intervals : max([length(br1_ext) length(br2_ext)])
    subplot(211);
    plot(br1_ext(a:a+num_intervals-1),'b');
    hold on;
    if a < length(br2_ext)
        plot(br2_ext(a:a+num_intervals-1),'r');
    end
    subplot(212);
    plot(br1_rea(a:a+num_intervals-1),'b');
    hold on;
    if a < length(br2_rea)
        plot(br2_rea(a:a+num_intervals-1),'r');
    end
end
subplot(211); title('pw extension - ch3');
subplot(212); title('pw reach - ch3');
legend({'B07 BR','B16 ND'});
fig.Position = [0 0 2480 3508];
if save_flag
    saveFig2PDF('.\Figures\', '_', 'pw_ch3',f);
end

f = f + 1;
sel_ch = [3 4 5 8 9 10];
figure(f);
X_br_ext(:,:) = iemg_struct.WT_W06_B07_BR.ext(:,sel_ch);
X_nd_ext(:,:) = iemg_struct.WT_W06_B16_ND.ext(:,sel_ch);
X_br_reach(:,:) = iemg_struct.WT_W06_B07_BR.reach(:,sel_ch);
X_nd_reach(:,:) = iemg_struct.WT_W06_B16_ND.reach(:,sel_ch);

for c = 1 : length(sel_ch)
    subplot(3,2,c);
    plot(X_br_ext(:,c), X_br_reach(:,c), 'b*',...
        X_nd_ext(:,c), X_nd_reach(:,c), 'r*');
    title(fields{sel_ch(c)});
    xlabel('extension iemg');
    ylabel('reaching iemg');
end

legend({'B07 BR','B16 ND'});
fig = figure(f);
fig.Position = [0 0 2480 3508];
if save_flag
    saveFig2PDF('.\Figures\', '_', 'iemg_cmp',f);
end

f = f + 1;
fig = figure(f);
Xpk_br_ext(:,:)   = pk2pk_struct.WT_W06_B07_BR.ext(:,sel_ch);
Xpk_nd_ext(:,:)   = pk2pk_struct.WT_W06_B16_ND.ext(:,sel_ch);
Xpk_br_reach(:,:) = pk2pk_struct.WT_W06_B07_BR.reach(:,sel_ch);
Xpk_nd_reach(:,:) = pk2pk_struct.WT_W06_B16_ND.reach(:,sel_ch);

for c = 1 : length(sel_ch)
    subplot(3,2,c);
    plot(Xpk_br_ext(:,c), Xpk_br_reach(:,c), 'b*',...
        Xpk_nd_ext(:,c), Xpk_nd_reach(:,c), 'r*');
    title(fields{sel_ch(c)});
    xlabel('extension pk2pk');
    ylabel('reaching pk2pk');
end
legend({'B07 BR','B16 ND'});

fig.Position = [0 0 2480 3508];
if save_flag
    saveFig2PDF('.\Figures\','_','pk2pk_cmp',f);
end

f = f + 1;
fig = figure(f);
for c = 1 : size(X_br_ext,2)
    subplot(3,2,c);
    plot(X_br_ext(:,c), Xpk_br_ext(:,c), 'b*', ...
        X_nd_ext(:,c), Xpk_nd_ext(:,c), 'r*');
    title(fields{sel_ch(c)});
    xlabel('iemg');
    ylabel('pk2pk');
end
legend({'B07 BR','B16 ND'});
fig.Position = [0 0 2480 3508];
if save_flag
    saveFig2PDF('.\Figures\','_','iemg_pk2pk',f);
end


end

function mu_arr = getNthRowMean(x, n)
    s = numel(x);
    xx = reshape(x(1 : s - mod(s,n)), n, []);
    arr = mean(xx,1);
    [nrows, ncols] = size(x);
    mu_arr = reshape(arr, nrows/n, ncols);
end

function sd_arr = getNthRowSD(x, n)
    s = numel(x);
    xx = reshape(x(1 : s - mod(s,n)), n, []);
    arr = std(xx);
    [nrows, ncols] = size(x);
    sd_arr = reshape(arr, nrows/n, ncols);
end
