function main()

close all;
mep_flag = false;
save_fig_flag = false;
plot_flag = true;

config();
load('config.mat');

% Initialise which directories to look at 
sel_dirs = {'.\W00', ...
            '.\W01'};

nmep_lim = 75;
nelec_ch = 5;
ch_names = {'Deltoid', 'Bicep', 'Pronator', 'Flexor', 'Extensor'};

% Screen and figure setup
% get the computer screen size
scrsz = get(0,'ScreenSize');

MEP_win_time = (-PRE_WINDOW/fs : 1/fs : POST_WINDOW/fs)*fs'; 
MEP_pre_time = (-PRE3_MEP/fs:1/fs:POST3_MEP/fs)*1000';

% Loop through all the selected directories
for nsel = 1 : length(sel_dirs)
    sel_dir = cell2mat(sel_dirs(nsel));
    addpath(genpath(sel_dir));
    file_list = dir(sel_dir);
    flist = {file_list.name};
    flist = flist(contains(flist,'.bin'));

    % If the selected directory contains 'intact' data then we perform a
    % baseline measurement protocol
    if contains(lower(sel_dir), 'intact')
        base_flag = true;
        base = struct();
    else
        base_flag = false;
    end

    %LOOP to process all files
    for n_file = 1:length(flist)
        % Open a bin file
        chosenfile = flist{n_file};

        fileinfo = getFileInfo(chosenfile);
        Week = fileinfo.Week;
        AnNum = fileinfo.AnNum;
        Anode = fileinfo.Anode;
        Treatment = fileinfo.Treatment;
        Cathode = fileinfo.Cathode;
        Current = fileinfo.Current;

        nrows = 6; % no of row
        ncols = 1; % no of column
        % calculate subplot position by calling subfigrid funtion
        pos = subfigrid(nrows,ncols,...
          [0.07 0.01 0.02 0.13],[0.85 0.85]); % [left Right Top Bottom]

        %~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        % Load EMG into data_array
        EMG_header = load_emg_header(chosenfile, 1);
        data_array = load_emg_data(chosenfile, ...
            EMG_header,EMG_header.data_position,1,inf,1);
        emg_data = data_array(:,1:nelec_ch);
        stim_pulse = data_array(:,6);

        %ZEROS
        % average MEPs
        mean_MEP = zeros(PRE_WINDOW+POST_WINDOW+1,nelec_ch);
        % rectified MEPs
        rec_MEP = zeros(PRE_WINDOW+POST_WINDOW+1,nelec_ch);
        % rectified MEPs for baseline
        rec_burst = zeros(PRE_WINDOW+1,nelec_ch);
        % response variables (peak to peak amplitude)
        early_amp = zeros(1,nelec_ch); 
        late_amp  = zeros(1,nelec_ch);
        % latency variables
        latency_early = zeros(1,nelec_ch);
        % area under rectified MEP responses
        auc_early = zeros(1,nelec_ch);
        auc_late = zeros(1, nelec_ch);

        % creating time vector
        inds = 1 : size(data_array,1);
        tend = length(data_array(:,STIM_CH))/fs;
        tvec = 0 : 1/fs : tend;
        tvec(1) = [];   % fs start from time zero 

        % Filter all data
        emg_filt = bpFilter(emg_data, 2, 30, 1e3, nq);
        [~, emg_tkeo] = energyop(emg_filt, false);
        emg_tkeo = reshape(emg_tkeo,size(emg_filt));
        emg_env = abs(lpFilterEMG(emg_tkeo, 2, 50, nq));

        % Select the type of data to be worked on. Either filt of
        % normalized to baseline. If this is not a baseline run, then we
        % select the normalized data as default.
        %{
        if ~base_flag
            emg_norm = normalizeEMGData(emg_filt, ...
                baseline_struct, AnNum, Anode, Cathode, Current);
            emg_ac = emg_norm;
        else
            emg_ac = emg_filt;
        end
        %}
        emg_ac = emg_filt;

        % Perform thresholding (mu + J*sd)
        thold = thresholding(emg_env, THR_MUL);
        active_mask = emg_env > thold;

        % map all the stim pulses based on the activity
        new_stim = map_stimpulse(active_mask, stim_pulse);

        % Knowing that all stim pulses are equal to 0, match them across
        % channels
        for c = 1 : nelec_ch - 1
            if c == 1
                final_stim = and(round(new_stim(:,c)), ...
                    round(new_stim(:,c+1)));
            else
                final_stim = and(final_stim, ...
                    round(new_stim(:,c+1)) );
            end
        end

        % Get number of registered MEPs
        session = length(final_stim);

        %PLOT1
        if plot_flag
            h1 = figure(1);
            titles = chosenfile(25:end-4);
            set(h1, 'name',titles,'Position', ...
                [1 scrsz(2) scrsz(3) scrsz(4)]);

            for c = 1 : nelec_ch
                % plot figure
                % subplot('position',pos(c,:,1)); 
                subplot(nelec_ch+1,1,c);
                plot(tvec, emg_ac(:,c), 'k');
                hold on;
                plot(tvec, active_mask(:,c), 'c');
                plot(tvec, emg_env(:,c)*5, 'g');
                plot([tvec(1) tvec(end)], [thold(c) thold(c)]*5, 'b');
                plot(tvec, new_stim(:,c), 'r', 'LineWidth', 1.5);
                xlim([1 length(tvec)/fs]); ylim([-1 1]);
                set(gca,'fontsize',16); 
                ylabel(['Ch ' num2str(c)],'fontsize',16);
            end
            subplot('position',pos(6,:,1)); 
            plot(tvec, final_stim, 'b', 'LineWidth', 1.5);
            xlim([1 session/fs]); ylim([-0.5 1.5]);
            set(gca,'fontsize',16); 
            xlabel('Sec','fontsize',16); ylabel('Ch 6','fontsize',16);
            if save_fig_flag
                saveas(h1, [Save_filepath, heading, '.tif']);
            end
        end

        % MEP responses here
        if mep_flag
            b_name = [AnNum '\'];
            dir_list = dir([mep_filepath b_name]);
            if length(dir_list) < 1
                mkdir([mep_filepath b_name]);
            end
            for c = 1 : 5
                s1 = sprintf('ch%d',c);
                csv_fname = sprintf('%s\\%s\\%s_%s', ...
                    mep_filepath,b_name,...
                    fname, s1);

                [iemg_temp , pk2pk_temp] = mepresponse(...
                    filtdata(:,c), ...
                    new_stim(:,c), fs, c);

                iemg(index, 1).Week = Week;
                iemg(index, 1).AnNum = AnNum;
                iemg(index, 1).Anode = Anode;
                iemg(index, 1).Cathode = Cathode;
                iemg(index, 1).Current = Current;
                iemg(index, 1).Group = gnum;
                iemg(index, 1).Channel = c;
                iemg(index, 1).MiddleResponses = iemg_temp.mr;
                iemg(index, 1).LateResponses = iemg_temp.lr;

                pk2pk(index, 1).Week = Week;
                pk2pk(index, 1).AnNum = AnNum;
                pk2pk(index, 1).Anode = Anode;
                pk2pk(index, 1).Cathode = Cathode;
                pk2pk(index, 1).Current = Current;
                pk2pk(index, 1).Group = gnum;
                pk2pk(index, 1).Channel = c;
                pk2pk(index, 1).MRMeans = pk2pk_temp.mr_means;
                pk2pk(index, 1).LRMeans = pk2pk_temp.lr_means;
                pk2pk(index, 1).MRPeak2Peak = pk2pk_temp.mr;
                pk2pk(index, 1).LRPeak2Peak = pk2pk_temp.lr;
                pk2pk(index, 1).MRNumPeaks = pk2pk_temp.mr_num_pks;
                pk2pk(index, 1).LRNumPeaks = pk2pk_temp.lr_num_pks;
                
                index = index + 1;
            end
        end

        % Get edges of mep stim pulses
        pulse_thold = SLOPE;
        pulse_offset = [final_stim(2:end); NaN];
        risingEdge = inds(final_stim < pulse_thold & ...
            pulse_offset > pulse_thold);

        counter = risingEdge';
        % Get MEP traces
        [meps, pk2pk, latency, iemg] = meptrace(emg_ac, counter);
        % Organised per MEP for each channel
        out_MEPs = meps.out_MEPs;
        mean_MEP = meps.mean_MEP;
        rec_MEP  = meps.rec_MEP;

        early_amp = pk2pk.early_pk2pk;
        late_amp  = pk2pk.late_pk2pk;

        latency_early = latency.early;
        latency_late = latency.early;

        auc_early = iemg.early;
        auc_late = iemg.late;
        clear out_MEPs;

        fprintf('%d/%d ... %s\n', n_file, length(flist), chosenfile);
    end
end
    
end
