function [meps, pk2pk, latency, iemg] = meptrace(emg_data, counter)
% Creates the traces from the given emg_data variable and performs pk2pk,
% latency and iemg calculations from the given mep traces.
% USAGE
% [meps, pk2pk, latency, iemg] = meptrace(emg_data, counter)
% emg_data = emg data organised by column [no. data points, no. of channels]
% counter = number of registered MEPs

% Load the configuration file for reference
load('config.mat');

% Get number of channels and initialise mean MEP and rectified MEP trace arrays
nelec_ch = size(emg_data,2);
mean_MEP = zeros(PRE_WINDOW+POST_WINDOW+1,nelec_ch);
rec_MEP = zeros(PRE_WINDOW+POST_WINDOW+1,nelec_ch);

% Initialise structs
meps = struct();
pk2pk = struct();
latency = struct();
iemg = struct();

% Get number of trials and data length
trials = length(counter)-1;  
dlen = length(emg_data);

% For each trial, separate the mep traces into a 3d array sorted by [mep trace
% data, no. of trials, no. of channels]
for j = 1 : trials
    k = counter(j);
    if k < PRE_WINDOW
        k = PRE_WINDOW + 1;
    elseif k+POST_WINDOW > dlen
        k = dlen - POST_WINDOW;
    end
    k_window = k - PRE_WINDOW : k + POST_WINDOW;
    mean_MEP = mean_MEP + emg_data(k_window,:);
    rec_MEP = rec_MEP + abs(emg_data(k_window, :));
    if k < PRE3_MEP
        k = PRE3_MEP + 1;
    elseif k+POST3_MEP > dlen
        k = dlen - POST3_MEP;
    end
    k_mep = k - PRE3_MEP + 1: k + POST3_MEP;
    out_MEPs(:,j,:) = emg_data(k_mep , :);
    hold all
end

% calculate mean
mean_MEP = mean_MEP/j;
rec_MEP = rec_MEP/j;

meps.out_MEPs = out_MEPs;
meps.mean_MEP = mean_MEP;
meps.rec_MEP = rec_MEP;

%below calculations are from data at 0-30msec after stimulation pulse onset
%average peak-to-peak responses
er = OFF+1:MEP_WINDOW;
lr = MEP_WINDOW + 1 : POST_WINDOW;
for j = 1 : trials
    trace = squeeze(out_MEPs(:,j,:));
    er_trace = trace(er,:);
    lr_trace = trace(lr,:);
    [~ , er_lat(j,:)] = max(abs(er_trace));
    [~ , lr_temp] = max(abs(lr_trace));
    lr_lat(j,:) =  (lr_temp + lr(1));
    er_pk2pk(j,:) = max(er_trace) - min(er_trace);
    lr_pk2pk(j,:) = max(lr_trace) - min(lr_trace);
end
er_lat = er_lat./10;
lr_lat = lr_lat./10;
early_pk2pk = max(mean_MEP(OFF+1:MEP_WINDOW, :)) - ...
    min(mean_MEP(OFF+1:MEP_WINDOW,:));
late_pk2pk = max(mean_MEP(MEP_WINDOW+1:POST_WINDOW,:)) - ...
    min(mean_MEP(MEP_WINDOW+1:POST_WINDOW,:));
pk2pk.early_pk2pk_vals = er_pk2pk;
pk2pk.late_pk2pk_vals = lr_pk2pk;
pk2pk.early_pk2pk = mean(er_pk2pk);
pk2pk.late_pk2pk  = mean(lr_pk2pk);
% latency calculation
for c = 1 : nelec_ch
    [Max_early(c) Imax_early(c)] = max(abs(mean_MEP(OFF+1:MEP_WINDOW,c)));
    [Max_late(c) Imax_late(c)] = max(abs(mean_MEP(MEP_WINDOW+1: ...
        POST_WINDOW,c)));
end
latency_early = Imax_early/10; %latency in ms
latency_late = (MEP_WINDOW+1 + Imax_late)/10; %latency in ms

latency.early = latency_early;
latency.late = latency_late;

%area under curve
iemg_early = trapz(rec_MEP(OFF+1:MEP_WINDOW,:));
iemg_late = trapz(rec_MEP(MEP_WINDOW+1:POST_WINDOW,:));
iemg.early = iemg_early;
iemg.late = iemg_late;
end
